@@ -80,12 +80,7 @@ int main(int argc, char* argv[])
     const int n = 2 < argc ? std::atoi(argv[2]) : m;
     const int k = 3 < argc ? std::atoi(argv[3]) : m;
 
-    const int csize = m * n;
-    if ((MAX_SIZE) < csize) {
-      throw std::runtime_error("The size M x N is exceeding MAX_SIZE!");
-    }
-
-    const int asize = m * k, bsize = k * n, aspace = LIBXSMM_ALIGNMENT / sizeof(T);
+    const int asize = m * k, bsize = k * n, csize = m * n, aspace = LIBXSMM_ALIGNMENT / sizeof(T);
     const int s = (2ULL << 30) / ((asize + bsize + csize) * sizeof(T)); // 2 GByte
     const size_t bwsize_batched = (asize/*load*/ + bsize/*load*/ + 2 * csize/*RFO*/) * sizeof(T); // batched
     const size_t bwsize = (asize/*load*/ + bsize/*load*/) * sizeof(T); // streamed, skipping C since it is just in cache
@@ -145,49 +140,54 @@ int main(int argc, char* argv[])
         fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
       }
 
-      { // streaming
-        fprintf(stdout, "Streamed (A,B)...\n");
-        const unsigned long long start = libxsmm_timer_tick();
+      if ((MAX_SIZE) >= csize) {
+        { // streaming
+          fprintf(stdout, "Streamed (A,B)...\n");
+          const unsigned long long start = libxsmm_timer_tick();
 #if defined(_OPENMP)
-#       pragma omp parallel for
+#         pragma omp parallel for
 #endif
-        for (int i = 0; i < s; ++i) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
-          // do nothing else with tmp; just a benchmark
-          libxsmm_gemm(0/*transa*/, 0/*transb*/, m, n, k,
-            0/*alpha*/, a + i * asize, 0/*lda*/, b + i * bsize, 0/*ldb*/,
-            0/*beta*/, tmp, 0/*ldc*/);
+          for (int i = 0; i < s; ++i) {
+            // make sure that stacksize is covering the problem size
+            T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
+            T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+            // do nothing else with tmp; just a benchmark
+            libxsmm_gemm(0/*transa*/, 0/*transb*/, m, n, k,
+              0/*alpha*/, a + i * asize, 0/*lda*/, b + i * bsize, 0/*ldb*/,
+              0/*beta*/, tmp, 0/*ldc*/);
+          }
+          const double duration = libxsmm_timer_duration(start, libxsmm_timer_tick());
+          if (0 < duration) {
+            fprintf(stdout, "\tperformance: %.1f GFLOPS/s\n", gflops / duration);
+            fprintf(stdout, "\tbandwidth: %.1f GB/s\n", s * bwsize / (duration * (1 << 30)));
+          }
+          fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
         }
-        const double duration = libxsmm_timer_duration(start, libxsmm_timer_tick());
-        if (0 < duration) {
-          fprintf(stdout, "\tperformance: %.1f GFLOPS/s\n", gflops / duration);
-          fprintf(stdout, "\tbandwidth: %.1f GB/s\n", s * bwsize / (duration * (1 << 30)));
-        }
-        fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
-      }
 
-      { // cached
-        fprintf(stdout, "Cached...\n");
-        const unsigned long long start = libxsmm_timer_tick();
+        { // cached
+          fprintf(stdout, "Cached...\n");
+          const unsigned long long start = libxsmm_timer_tick();
 #if defined(_OPENMP)
-#       pragma omp parallel for
+#         pragma omp parallel for
 #endif
-        for (int i = 0; i < s; ++i) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
-          // do nothing else with tmp; just a benchmark
-          libxsmm_gemm(0/*transa*/, 0/*transb*/, m, n, k,
-            0/*alpha*/, a, 0/*lda*/, b, 0/*ldb*/,
-            0/*beta*/, tmp, 0/*ldc*/);
+          for (int i = 0; i < s; ++i) {
+            // make sure that stacksize is covering the problem size
+            T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
+            T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+            // do nothing else with tmp; just a benchmark
+            libxsmm_gemm(0/*transa*/, 0/*transb*/, m, n, k,
+              0/*alpha*/, a, 0/*lda*/, b, 0/*ldb*/,
+              0/*beta*/, tmp, 0/*ldc*/);
+          }
+          const double duration = libxsmm_timer_duration(start, libxsmm_timer_tick());
+          if (0 < duration) {
+            fprintf(stdout, "\tperformance: %.1f GFLOPS/s\n", gflops / duration);
+          }
+          fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
         }
-        const double duration = libxsmm_timer_duration(start, libxsmm_timer_tick());
-        if (0 < duration) {
-          fprintf(stdout, "\tperformance: %.1f GFLOPS/s\n", gflops / duration);
-        }
-        fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
+      }
+      else {
+        fprintf(stderr, "Warning: size M x N is exceeding MAX_SIZE!\n");
       }
 
       // finalize LIBXSMM
