@@ -90,8 +90,10 @@
         INTEGER(C_INT), PARAMETER ::                                    &
           ! No prefetching and no prefetch function signature.
      &    LIBXSMM_PREFETCH_NONE       = 0,                              &
+          ! Automatically select strategy (frontend).
+     &    LIBXSMM_PREFETCH_AUTO       = -1,                             &
           ! Only function prefetch signature.
-     &    LIBXSMM_PREFETCH_SIGNATURE  = 1,                              &
+     &    LIBXSMM_PREFETCH_SIGONLY    = 1,                              &
           ! Prefetch PA using accesses to A.
      &    LIBXSMM_PREFETCH_AL2        = 2,                              &
           ! Prefetch PA (aggressive).
@@ -233,8 +235,12 @@
           MODULE PROCEDURE libxsmm_blas_sgemm, libxsmm_blas_dgemm
         END INTERFACE
 
-        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_init, libxsmm_finalize, libxsmm_get_target_arch
-        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_timer_tick, libxsmm_timer_duration
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_init, libxsmm_finalize
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_get_target_arch
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_timer_duration
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_timer_tick
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_omp_sgemm
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_omp_dgemm
         INTERFACE
           ! Initialize the library; pay for setup cost at a specific point.
           SUBROUTINE libxsmm_init() BIND(C)
@@ -265,9 +271,47 @@
             IMPORT :: C_LONG_LONG, C_DOUBLE
             INTEGER(C_LONG_LONG), INTENT(IN), VALUE :: tick0, tick1
           END FUNCTION
-        END INTERFACE$MNK_INTERFACE_LIST
+
+          SUBROUTINE libxsmm_omp_sgemm(transa, transb, m, n, k,         &
+     &    alpha, a, lda, b, ldb, beta, c, ldc) BIND(C)
+            IMPORT LIBXSMM_BLASINT_KIND, C_CHAR, C_FLOAT
+            CHARACTER(C_CHAR), INTENT(IN) :: transa, transb
+            INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: m, n, k
+            INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: lda, ldb, ldc
+            REAL(C_FLOAT), INTENT(IN) :: alpha, beta
+            REAL(C_FLOAT), INTENT(IN) :: a(lda,*), b(ldb,*)
+            REAL(C_FLOAT), INTENT(INOUT) :: c(ldc,*)
+          END SUBROUTINE
+
+          SUBROUTINE libxsmm_omp_dgemm(transa, transb, m, n, k,         &
+     &    alpha, a, lda, b, ldb, beta, c, ldc) BIND(C)
+            IMPORT LIBXSMM_BLASINT_KIND, C_CHAR, C_DOUBLE
+            CHARACTER(C_CHAR), INTENT(IN) :: transa, transb
+            INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: m, n, k
+            INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: lda, ldb, ldc
+            REAL(C_DOUBLE), INTENT(IN) :: alpha, beta
+            REAL(C_DOUBLE), INTENT(IN) :: a(lda,*), b(ldb,*)
+            REAL(C_DOUBLE), INTENT(INOUT) :: c(ldc,*)
+          END SUBROUTINE
+      END INTERFACE$MNK_INTERFACE_LIST
 
       CONTAINS
+        ! Returns a name for the target architecture as identified
+        ! by libxsmm_get_target_arch().
+        !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_get_target_archid
+        PURE FUNCTION libxsmm_get_target_archid() RESULT(name)
+          CHARACTER(LEN=3) :: name
+          !DIR$ ATTRIBUTES OFFLOAD:MIC :: get_target_archid
+          INTERFACE
+            PURE SUBROUTINE get_target_archid(name, length) BIND(C)
+              IMPORT :: C_CHAR, C_INT
+              CHARACTER(C_CHAR), INTENT(OUT) :: name(*)
+              INTEGER(C_INT), VALUE, INTENT(IN) :: length
+            END SUBROUTINE
+          END INTERFACE
+          CALL get_target_archid(name, LEN(name))
+        END FUNCTION
+
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: srealptr
         FUNCTION srealptr(a) RESULT(p)
           REAL(C_FLOAT), INTENT(IN), TARGET :: a(:,:)
@@ -554,7 +598,7 @@
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_sgemm
         SUBROUTINE libxsmm_sgemm(transa, transb, m, n, k,               &
      &  alpha, a, lda, b, ldb, beta, c, ldc)
-          CHARACTER(1), INTENT(IN), OPTIONAL :: transa, transb
+          CHARACTER, INTENT(IN), OPTIONAL :: transa, transb
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: m, n, k
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: lda
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: ldb
@@ -583,7 +627,7 @@
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_dgemm
         SUBROUTINE libxsmm_dgemm(transa, transb, m, n, k,               &
      &  alpha, a, lda, b, ldb, beta, c, ldc)
-          CHARACTER(1), INTENT(IN), OPTIONAL :: transa, transb
+          CHARACTER, INTENT(IN), OPTIONAL :: transa, transb
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: m, n, k
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: lda
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: ldb
@@ -612,7 +656,7 @@
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_blas_sgemm
         SUBROUTINE libxsmm_blas_sgemm(transa, transb, m, n, k,          &
      &  alpha, a, lda, b, ldb, beta, c, ldc)
-          CHARACTER(1), INTENT(IN), OPTIONAL :: transa, transb
+          CHARACTER, INTENT(IN), OPTIONAL :: transa, transb
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: m, n, k
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: lda
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: ldb
@@ -641,7 +685,7 @@
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_blas_dgemm
         SUBROUTINE libxsmm_blas_dgemm(transa, transb, m, n, k,          &
      &  alpha, a, lda, b, ldb, beta, c, ldc)
-          CHARACTER(1), INTENT(IN), OPTIONAL :: transa, transb
+          CHARACTER, INTENT(IN), OPTIONAL :: transa, transb
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN) :: m, n, k
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: lda
           INTEGER(LIBXSMM_BLASINT_KIND), INTENT(IN), OPTIONAL :: ldb
