@@ -44,6 +44,18 @@
 # pragma offload_attribute(pop)
 #endif
 
+#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE /*|| any other implemented size*/)
+# if (LIBXSMM_X86_AVX512 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#   define LIBXSMM_GEMM_DIFF_AVX512
+# elif (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#   define LIBXSMM_GEMM_DIFF_AVX2
+# elif (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#   define LIBXSMM_GEMM_DIFF_AVX
+# else
+#   define LIBXSMM_GEMM_DIFF_NOWARNING
+# endif
+#endif
+
 
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL libxsmm_gemm_diff_function internal_gemm_diff_function = libxsmm_gemm_diff_sw;
 LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL libxsmm_gemm_diffn_function internal_gemm_diffn_function = libxsmm_gemm_diffn_sw;
@@ -119,14 +131,14 @@ unsigned int libxsmm_gemm_diff_sw(const libxsmm_gemm_descriptor* reference, cons
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_GEMM_DIFF_AVX)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(8 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(0 != reference && 0 != desc);
   {
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     int r0, r1;
     __m256i a256, b256;
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE) /* otherwise generate a compile-time error */
 #   if defined(__CYGWIN__) && !defined(NDEBUG) /* Cygwin/GCC: _mm256_set_epi32 may cause an illegal instruction */
     const union { int32_t array[8]; __m256i i; } m256 = { /* use literal value rather than yes/no
       in order to avoid warning about "initializer element is not computable at load time" */
@@ -137,23 +149,23 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
     struct { __m256i i; } m256;
     m256.i = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   endif
-# endif
-# if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
+#   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
     a256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)reference, m256.i));
-# else
+#   else
     /*a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
     a256 = _mm256_loadu_si256((const __m256i*)reference);
-# endif
+#   endif
     b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)desc, m256.i));
     r0 = _mm256_testnzc_si256(a256, b256);
     r1 = _mm256_testnzc_si256(b256, a256);
     return r0 | r1;
+# endif
   }
 #else
-# if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
   static LIBXSMM_TLS int once = 0;
   if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX instruction code path!\n");
+    fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
     once = 1;
   }
 # endif
@@ -165,31 +177,31 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_GEMM_DIFF_AVX2)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(8 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(0 != reference && 0 != desc);
   {
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE) /* otherwise generate a compile-time error */
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
-# endif
-# if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
+#   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
     const __m256i a256 = _mm256_maskload_epi32((const void*)reference, m256);
-# else
+#   else
     /*const __m256i a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
-# endif
+#   endif
     const __m256i b256 = _mm256_maskload_epi32((const void*)desc, m256);
     const int r0 = _mm256_testnzc_si256(a256, b256);
     const int r1 = _mm256_testnzc_si256(b256, a256);
     return r0 | r1;
+# endif
   }
 #else
-# if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
   static LIBXSMM_TLS int once = 0;
   if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX2 instruction code path!\n");
+    fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
     once = 1;
   }
 # endif
@@ -201,7 +213,7 @@ unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, co
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
 unsigned int libxsmm_gemm_diff_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
-#if defined(__MIC__)
+#if defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE/*|| any other implemented size*/)
   assert(0 ==  LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(16 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   assert(0 != reference && 0 != desc);
@@ -252,8 +264,7 @@ unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, con
   for (i = hint; i < end; ++i) {
     const unsigned int j = i % ndescs; /* wrap around index */
     /* negative stride runs backwards */
-    const char *const d = desc + j * nbytes;
-    if (0 == libxsmm_gemm_diff(reference, (const libxsmm_gemm_descriptor*)d)) {
+    if (0 == libxsmm_gemm_diff(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
       return j;
     }
   }
@@ -261,8 +272,7 @@ unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, con
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
   for (i = hint; i < end; ++i) {
     const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
-    const char *const d = desc + j * nbytes;
-    if (0 == libxsmm_gemm_diff(reference, (const libxsmm_gemm_descriptor*)d)) {
+    if (0 == libxsmm_gemm_diff(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
       return j;
     }
   }
@@ -275,15 +285,15 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_GEMM_DIFF_AVX)
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
   assert(32 == nbytes); /* padded descriptor array */
   {
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const unsigned int end = hint + ndescs;
     const char *const desc = (const char*)descs;
     __m256i a256;
     unsigned int i;
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE) /* otherwise generate a compile-time error */
 #   if defined(__CYGWIN__) && !defined(NDEBUG) /* Cygwin/GCC: _mm256_set_epi32 may cause an illegal instruction */
     const union { int32_t array[8]; __m256i i; } m256 = { /* use literal value rather than yes/no
       in order to avoid warning about "initializer element is not computable at load time" */
@@ -294,31 +304,31 @@ unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, co
     struct { __m256i i; } m256;
     m256.i = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   endif
-# endif
-# if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
+#   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
     a256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)reference, m256.i));
-# else
+#   else
     /*a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
     a256 = _mm256_loadu_si256((const __m256i*)reference);
-# endif
+#   endif
     for (i = hint; i < end; ++i) {
       const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
-#if defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
+#   if defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
       const __m256i b256 = _mm256_loadu_si256((const __m256i*)(desc + j * nbytes));
-#else
+#   else
       const __m256i b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)(desc + j * nbytes), m256.i));
-#endif
+#   endif
       if (0 == _mm256_testnzc_si256(a256, b256) && 0 == _mm256_testnzc_si256(b256, a256)) {
         return j;
       }
     }
+# endif
   }
   return ndescs;
 #else
-# if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
   static LIBXSMM_TLS int once = 0;
   if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX instruction code path!\n");
+    fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
     once = 1;
   }
 # endif
@@ -331,41 +341,41 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_GEMM_DIFF_AVX2)
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
   assert(32 == nbytes); /* padded descriptor array */
   {
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const unsigned int end = hint + ndescs;
     const char *const desc = (const char*)descs;
     unsigned int i;
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE) /* otherwise generate a compile-time error */
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
-# endif
-# if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
+#   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
     const __m256i a256 = _mm256_maskload_epi32((const void*)reference, m256);
-# else
+#   else
     /*const __m256i a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
-#endif
+#   endif
     for (i = hint; i < end; ++i) {
       const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
-#if defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
+#   if defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
       const __m256i b256 = _mm256_loadu_si256((const __m256i*)(desc + j * nbytes));
-#else
+#   else
       const __m256i b256 = _mm256_maskload_epi32((const void*)(desc + j * nbytes), m256);
-#endif
+#   endif
       if (0 == _mm256_testnzc_si256(a256, b256) && 0 == _mm256_testnzc_si256(b256, a256)) {
         return j;
       }
     }
+# endif
   }
   return ndescs;
 #else
-# if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
   static LIBXSMM_TLS int once = 0;
   if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX2 instruction code path!\n");
+    fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
     once = 1;
   }
 # endif
@@ -378,24 +388,23 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
 unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX512 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_GEMM_DIFF_AVX512)
   assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
   assert(32 == nbytes); /* padded descriptor array */
   {
+# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
     const unsigned int hint_even = (hint & 0xFFFFFFFE), end = hint_even + ndescs;
     const char *const desc = (const char*)descs;
     unsigned int i;
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE) /* otherwise generate a compile-time error */
     const __mmask16 mask_lo = 0x7F, mask_hi = 0x7F00;
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
-# endif
-# if defined(LIBXSMM_GEMM_DIFF_MASK_A)
+#   if defined(LIBXSMM_GEMM_DIFF_MASK_A)
     const __m256i a256 = _mm256_maskload_epi32((const void*)reference, m256);
-# else
+#   else
     /* SKX: consider _mm256_maskz_expandloadu_epi32 */
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
-#endif
+#   endif
     const __m512i a512 = _mm512_broadcast_i64x4(a256);
     for (i = hint_even; i < end; i += 2) {
       const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
@@ -408,13 +417,14 @@ unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference,
         return j + 1;
       }
     }
+# endif
   }
   return ndescs;
 #else
-# if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) && !defined(LIBXSMM_GEMM_DIFF_NOWARNING) /* library code is expected to be mute */
   static LIBXSMM_TLS int once = 0;
   if (0 == once) {
-    fprintf(stderr, "LIBXSMM: unable to enter AVX-512 instruction code path!\n");
+    fprintf(stderr, "LIBXSMM: unable to enter AVX-512 code path!\n");
     once = 1;
   }
 # endif
@@ -427,16 +437,14 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
 unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(__MIC__) && 0
+#if defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE/*|| any other implemented size*/) && 0/*disabled*/
   assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
   assert(32 == nbytes); /* padded descriptor array */
   {
     const unsigned int hint_even = (hint & 0xFFFFFFFE), end = hint_even + ndescs;
     const char *const desc = (const char*)descs;
     unsigned int i;
-# if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE) /* otherwise generate a compile-time error */
     const __mmask16 mask_lo = 0x7F, mask_hi = 0x7F00;
-# endif
     const __m512i h512 = _mm512_mask_loadunpackhi_epi32(
       _mm512_mask_loadunpacklo_epi32(_mm512_set1_epi32(0), mask_lo, reference),
       mask_lo, ((const char*)reference) + 32);
